import os

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.migration_abc import MigrationABC
from bot_data.db_context import DBContext


class ConfigMigration(MigrationABC):
    name = "1.1.0_ConfigMigration"

    def __init__(self, logger: DatabaseLogger, db: DBContext):
        MigrationABC.__init__(self)
        self._logger = logger
        self._db = db
        self._cursor = db.cursor

    def _exec(self, file: str):
        path = f"{os.path.dirname(os.path.realpath(__file__))}/db_history_scripts"
        sql = open(f"{path}/{file}").read()

        for statement in sql.split("\n\n"):
            self._cursor.execute(statement + ";")

    def upgrade(self):
        self._logger.debug(__name__, "Running upgrade")
        self._server_upgrade()
        self._technician_upgrade()

        self._exec("config/server.sql")
        self._exec("config/server_afk_channels.sql")
        self._exec("config/server_team_roles.sql")
        self._exec("config/technician.sql")
        self._exec("config/technician_ids.sql")
        self._exec("config/technician_ping_urls.sql")

    def _server_upgrade(self):
        self._cursor.execute(
            str(
                f"""
                    CREATE TABLE IF NOT EXISTS `CFG_Server` (
                        `Id` BIGINT NOT NULL AUTO_INCREMENT,
                        `MessageDeleteTimer` BIGINT NOT NULL DEFAULT 6,
                        `NotificationChatId` BIGINT NOT NULL,
                        `MaxVoiceStateHours` BIGINT NOT NULL DEFAULT 6,
                        `XpPerMessage` BIGINT NOT NULL DEFAULT 1,
                        `XpPerReaction` BIGINT NOT NULL DEFAULT 1,
                        `MaxMessageXpPerHour` BIGINT NOT NULL DEFAULT 20,
                        `XpPerOntimeHour` BIGINT NOT NULL DEFAULT 10,
                        `XpPerEventParticipation` BIGINT NOT NULL DEFAULT 10,
                        `XpPerAchievement` BIGINT NOT NULL DEFAULT 10,
                        `AFKCommandChannelId` BIGINT NOT NULL,
                        `HelpVoiceChannelId` BIGINT NOT NULL,
                        `TeamChannelId` BIGINT NOT NULL,
                        `LoginMessageChannelId` BIGINT NOT NULL,
                        `ServerId` BIGINT NOT NULL,
                        `CreatedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6),
                        `LastModifiedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6) ON UPDATE CURRENT_TIMESTAMP(6),
                        PRIMARY KEY(`Id`),
                        FOREIGN KEY (`ServerId`) REFERENCES `Servers`(`ServerId`)
                    );
                """
            )
        )

        self._cursor.execute(
            str(
                f"""
                    CREATE TABLE IF NOT EXISTS `CFG_ServerAFKChannelIds` (
                        `Id` BIGINT NOT NULL AUTO_INCREMENT,
                        `ChannelId` BIGINT NOT NULL,
                        `ServerId` BIGINT NOT NULL,
                        `CreatedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6),
                        `LastModifiedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6) ON UPDATE CURRENT_TIMESTAMP(6),
                        PRIMARY KEY(`Id`),
                        FOREIGN KEY (`ServerId`) REFERENCES `Servers`(`ServerId`)
                    );
                """
            )
        )

        self._cursor.execute(
            str(
                f"""
                    CREATE TABLE IF NOT EXISTS `CFG_ServerTeamRoleIds` (
                        `Id` BIGINT NOT NULL AUTO_INCREMENT,
                        `RoleId` BIGINT NOT NULL,
                        `TeamMemberType` ENUM('Moderator', 'Admin') NOT NULL,
                        `ServerId` BIGINT NOT NULL,
                        `CreatedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6),
                        `LastModifiedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6) ON UPDATE CURRENT_TIMESTAMP(6),
                        PRIMARY KEY(`Id`),
                        FOREIGN KEY (`ServerId`) REFERENCES `Servers`(`ServerId`)
                    );
                """
            )
        )

    def _technician_upgrade(self):
        self._cursor.execute(
            str(
                f"""
                    CREATE TABLE IF NOT EXISTS `CFG_Technician` (
                        `Id` BIGINT NOT NULL AUTO_INCREMENT,
                        `HelpCommandReferenceUrl` VARCHAR(255) NOT NULL,
                        `WaitForRestart` BIGINT NOT NULL DEFAULT 8,
                        `WaitForShutdown` BIGINT NOT NULL DEFAULT 8,
                        `CacheMaxMessages` BIGINT NOT NULL DEFAULT 1000000,
                        `CreatedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6),
                        `LastModifiedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6) ON UPDATE CURRENT_TIMESTAMP(6),
                        PRIMARY KEY(`Id`)
                    );
                """
            )
        )

        self._cursor.execute(
            str(
                f"""
                    CREATE TABLE IF NOT EXISTS `CFG_TechnicianPingUrls` (
                        `Id` BIGINT NOT NULL AUTO_INCREMENT,
                        `URL` VARCHAR(255) NOT NULL,
                        `CreatedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6),
                        `LastModifiedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6) ON UPDATE CURRENT_TIMESTAMP(6),
                        PRIMARY KEY(`Id`)
                    );
                """
            )
        )

        self._cursor.execute(
            str(
                f"""
                    CREATE TABLE IF NOT EXISTS `CFG_TechnicianIds` (
                        `Id` BIGINT NOT NULL AUTO_INCREMENT,
                        `TechnicianId` BIGINT NOT NULL,
                        `CreatedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6),
                        `LastModifiedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6) ON UPDATE CURRENT_TIMESTAMP(6),
                        PRIMARY KEY(`Id`)
                    );
                """
            )
        )

    def downgrade(self):
        self._logger.debug(__name__, "Running downgrade")

    def _server_downgrade(self):
        self._cursor.execute("DROP TABLE `CFG_Server`;")

    def _technician_downgrade(self):
        self._cursor.execute("DROP TABLE `CFG_Technician`;")
        self._cursor.execute("DROP TABLE `CFG_TechnicianPingUrls`;")
        self._cursor.execute("DROP TABLE `CFG_TechnicianIds`;")
